/**
 * Copyright (c) 2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

const _ = require('lodash');

function stringOfLength(string, length) {
  let newString = '';
  for (let i = 0; i < length; i++) {
    newString += string;
  }
  return newString;
}

function generateTitle(name, level) {
  const title = '`' + name + '` (component)';
  return stringOfLength('#', level) + ' ' + title + '\n'
}

function generateDesciption(description) {
  return description + '\n';
}

function generatePropType(type) {
  let values;
  if (Array.isArray(type.value)) {
    values =
      '(' +
      type.value
        .map(function(typeValue) {
          return typeValue.name || typeValue.value;
        })
        .join('|') +
      ')';
  } else {
    values = type.value;
  }

  return 'type: `' + type.name + (values ? values : '') + '`\n';
}

function generatePropDefaultValue(value) {
  return 'defaultValue: `' + value.value + '`\n';
}

function generateProp(propName, prop, level) {
  return (
    stringOfLength('#', level) + ' `' +
    propName +
    '`' +
    (prop.required ? ' (required)' : '') +
    '\n' +
    '\n' +
    (prop.description ? prop.description + '\n\n' : '') +
    (prop.type ? generatePropType(prop.type) : '') +
    (prop.defaultValue ? generatePropDefaultValue(prop.defaultValue) : '') +
    '\n'
  );
}

function generateProps(props, level) {
  const title = 'Props';
  if (!props) {
    return 'TBD';
  }

  return (
    stringOfLength('#', level) + ' ' +
    title +
    '\n' +
    '\n' +
    Object.keys(props)
      .sort()
      .map(function(propName) {
        return generateProp(propName, props[propName], level + 1);
      })
      .join('\n')
  );
}

function generateMarkdown(name, reactAPI, level = 1) {
  const markdownString =
    generateTitle(name, level) +
    '\n' +
    generateDesciption(reactAPI.description) +
    '\n' +
    generateProps(reactAPI.props, level + 1);

  return markdownString;
}

module.exports = generateMarkdown;
